package com.sharecare.config;

import com.amazonaws.auth.AWSStaticCredentialsProvider;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * AWS configuration class
 *
 * Starts the connection with S3.
 */
@Configuration
@Setter
@Getter
public class AWSConfig {

    @Value("${aws.key}")
    private String awsKey;
    @Value("${aws.secret}")
    private String awsSecret;

    @Bean
    public AmazonS3 amazons3() {

        AmazonS3 s3 = AmazonS3ClientBuilder
                .standard()
                .withRegion(Regions.EU_WEST_1)
                .withCredentials(new AWSStaticCredentialsProvider( new BasicAWSCredentials(awsKey, awsSecret)))
                .build();

        return s3;
    }
}
