package com.sharecare.core.aws;

import com.amazonaws.HttpMethod;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AccessControlList;
import com.amazonaws.services.s3.model.CannedAccessControlList;
import com.amazonaws.services.s3.model.ObjectMetadata;
import com.amazonaws.services.s3.model.PutObjectRequest;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Collections;
import java.util.Date;
import java.util.UUID;

/**
 * Class to deal with document upload and download from WS S3
 */
@Service
@RequiredArgsConstructor
public class DocumentService {

    private final AmazonS3 s3;
    private final static String BUCKET_NAME = "sharecare";


    /**
     * Method to get the link to a file stored in S3
     *
     * @param key Key of the object stored in BUCKET_NAME bucket in S3
     * @return
     */
    public URL downloadLink(@NonNull String key) {
        return s3.getUrl(BUCKET_NAME, key);
    }

    /**
     * Method to upload a document into the BUCKET_NAME bucket in S3.
     * The document uploaded has public access for views.
     *
     * @param inputStream content of the file
     * @param originalFilename
     * @param size
     * @param contentType
     * @return
     */
    public String uploadDocument(InputStream inputStream, String originalFilename, long size, String contentType) {
        ObjectMetadata metadata = new ObjectMetadata();
        metadata.setContentLength(size);
        metadata.setContentType(contentType);
        metadata.setUserMetadata(Collections.singletonMap("originalFilename", originalFilename));

        String key = UUID.randomUUID().toString();
        s3.putObject(BUCKET_NAME, key, inputStream, metadata);
        s3.setObjectAcl(BUCKET_NAME, key, CannedAccessControlList.PublicRead);
        return key;

    }
}
