package com.sharecare.webapp.config;

import com.sharecare.webapp.service.AccountInterceptor;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

@Configuration
@RequiredArgsConstructor
@EnableWebMvc
public class WebMvcConfiguration extends WebMvcConfigurerAdapter {

    private static final String STATIC_PATH_MATCHER = "/static/**";

    @Value("${web.staticResourceLocations}")
    private String staticResourceLocations;
    @Value("${web.httpCacheTime:1000}")
    private int cachePeriod;

    private final AccountInterceptor accountInterceptor;

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler(STATIC_PATH_MATCHER)
                .setCachePeriod(cachePeriod)
                .addResourceLocations(staticResourceLocations);
        registry.setOrder(5);
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        super.addInterceptors(registry);
        registry.addInterceptor(accountInterceptor);
    }

}
