package com.sharecare.webapp.config;

import com.sharecare.webapp.service.AuthenticationProviderProxy;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

/**
 * Class for web app security configuration.
 *
 * Allows user to keep a session opened and sets boundaries and permissions.
 */
@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
public class WebappSecurityConfiguration extends WebSecurityConfigurerAdapter {

    private final AuthenticationProviderProxy authenticationProviderProxy;

    public void configure(AuthenticationManagerBuilder authenticationManagerBuilder) throws Exception {
        authenticationManagerBuilder
                .authenticationProvider(authenticationProviderProxy)
                .eraseCredentials(true);
    }

    public void configure(HttpSecurity http) throws Exception {
        http
                .authorizeRequests()
                .antMatchers("/login").permitAll()
                .antMatchers("/join").permitAll()
                .antMatchers("/app/**", "/app").authenticated()
                .and()
                .formLogin()
                .loginPage("/login").permitAll()
                .failureUrl("/")
                .defaultSuccessUrl("/app")
                .and()
                .csrf()
                .requireCsrfProtectionMatcher((httpServletRequest -> !httpServletRequest.getRequestURI().equals("/login") &&
                        !(httpServletRequest.getMethod().toLowerCase().equals("get") || httpServletRequest.getMethod().toLowerCase().equals("head") || httpServletRequest.getMethod().toLowerCase().equals("options"))))
                .and()
                .logout()
                .logoutRequestMatcher(new AntPathRequestMatcher("/logout"))
                .logoutSuccessUrl("/login")
                .permitAll()
        ;

    }

}
