package com.sharecare.webapp.mvc;

import com.sharecare.core.aws.DocumentService;
import com.sharecare.core.like.LikeService;
import com.sharecare.core.searchable.SearchService;
import com.sharecare.core.searchable.SearchableEntity;
import com.sharecare.core.*;
import com.sharecare.webapp.service.UserSessionService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * Main controller of the webapp.
 */
@Controller
@RequestMapping("/app")
@RequiredArgsConstructor
public class AppController {

    private final UserSessionService userSessionService;
    private final UserRepository userRepository;

    private final QuestionRepository questionRepository;
    private final PostService postService;
    private final AnswerRepository answerRepository;
    private final ViewCountService viewCountService;
    private final LikeService likeService;

    private final SearchService searchService;
    private final DocumentService documentService;


    @PostMapping("/upload")
    public String uploadDoc(@RequestParam("doc") MultipartFile doc) throws IOException {

        documentService.uploadDocument(doc.getInputStream(), doc.getOriginalFilename(), doc.getSize(), doc.getContentType());

        return "redirect:/login";
    }

    @GetMapping
    public String getAccount(Model model) {
        return "account";
    }

    @GetMapping("/profile")
    public String getProfile(Model model) {
        //Optional<User> userOptional = userSessionService.sessionUser();
        //model.addAttribute("user", userOptional.get());

        return "profile";
    }

    @PostMapping("/settings")
    public String changeUserData(@RequestParam("name") String name) {
        Optional<User> userOptional = userSessionService.sessionUser();
        if (userOptional.isPresent()) {
            User u = userOptional.get();
            u.setName(name);
            userRepository.save(u);

            return "redirect:/app";
        }
            return "redirect:/";
    }

    @GetMapping("/questions")
    public String getQuestions() {

        return "questions";
    }

    @GetMapping("/new-question")
    public String newQuestion() {

        return "new-question";
    }

    @GetMapping("/download")
    public String getDoc(@RequestParam("docName") String docName) {
        URL presignedURL = documentService.downloadLink(docName);
        return "redirect:"+presignedURL;
    }

    @GetMapping("/question/{questionId}")
    public String getQuestion(@PathVariable("questionId") long questionId, Model model) {

        Question q = questionRepository.findOne(questionId);
        User u = userSessionService.sessionUser().get();
        model.addAttribute("viewCount", viewCountService.getViewCount(q));
        model.addAttribute("question", q);
        model.addAttribute("likes", likeService.getLikes(q));
        model.addAttribute("dislikes", likeService.getDislikes(q));
        viewCountService.incrViewCountFor(q);

        if (q.getDocKey() != null) {
            model.addAttribute("docUrl", documentService.downloadLink(q.getDocKey()));
        }

        model.addAttribute("hasVoted", likeService.getVote(u, q));
        return "question";
    }

    @PostMapping("/postquestion")
    public String postQuestion (@RequestParam("title") String title,
                                @RequestParam("bodyUnformatted") String bodyUnformatted,
                                @RequestParam("question") String question,
                                @RequestParam("doc") MultipartFile doc) throws IOException {

        String docKey = null;
        if (doc != null && !doc.isEmpty()){
            docKey = documentService.uploadDocument(doc.getInputStream(), doc.getOriginalFilename(), doc.getSize(), doc.getContentType());
        }
        Question q = postService.createQuestion(title, question, bodyUnformatted, docKey, doc.getContentType(), doc.getOriginalFilename(), userSessionService.sessionUser().get());
        searchService.makeSearchable(q);
        viewCountService.createViewCountFor(q);
        likeService.initLikes(q);

        return "redirect:/app";
    }

    @PostMapping("/{questionId}/postanswer")
    public String postAnswer (@RequestParam("bodyUnformatted") String bodyUnformatted,
                                @RequestParam("answer") String answer,
                              @PathVariable("questionId") long questionId) {

        Answer a = postService.createAnswer(answer, bodyUnformatted, userSessionService.sessionUser().get(), questionId);
        searchService.makeSearchable(a);

        return "redirect:/app";
    }



    @GetMapping("/question/search")
    public String search (@RequestParam("search") String search, Model model) {

        List<SearchableEntity> searchableEntities = searchService.searchEntities(search, 0);
        List<Question> questionList = new ArrayList<>();
        searchableEntities.stream()
                .forEach(a -> {
                    if (a instanceof Answer) {
                        if (!questionList.contains(((Answer) a).getQuestion())) {
                            questionList.add(((Answer) a).getQuestion());
                        }
                    } else if (a instanceof Question){
                        questionList.add((Question) a);
                    }
                });

        model.addAttribute("questions", questionList);
        return "find-questions";

    }

    /*
    //todo: this is obviously not alright like this
    @PostMapping("/addlike/{questionId}")
    public String addLike(@PathVariable("questionId") long questionId, Model model) {
        Question q = questionRepository.findOne(questionId);
        User u = userSessionService.sessionUser().get();


        return "redirect:/app/question/" + questionId;
    }*/


}
