package com.sharecare.webapp.mvc;

import com.sharecare.core.Question;
import com.sharecare.core.QuestionRepository;
import com.sharecare.core.User;
import com.sharecare.core.like.LikeService;
import com.sharecare.core.like.Vote;
import com.sharecare.webapp.service.UserSessionService;
import lombok.RequiredArgsConstructor;
import lombok.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RequiredArgsConstructor
@Controller
@RequestMapping("/app/question/{questionId}")
public class LikeController {

    private final QuestionRepository questionRepository;
    private final UserSessionService userSessionService;
    private final LikeService likeService;

    @PostMapping(value = "/like", produces = "application/json")
    @ResponseBody
    public VoteStatus addLike(@PathVariable("questionId") long questionId) {

        Question q = questionRepository.findOne(questionId);
        User u = userSessionService.sessionUser().get();

        Optional<Vote> vote = likeService.getVote(u, q);
        if (vote.isPresent()) {
            likeService.undoLikeOrDislike(u, q);
            if (vote.get().isDislike()) {
                likeService.addLike(u, q);
            }
        } else {
            likeService.addLike(u, q);
        }
        return getVoteStatus(q);
    }

    @PostMapping(value = "/dislike", produces = "application/json")
    @ResponseBody
    public VoteStatus addDislike(@PathVariable("questionId") long questionId) {
        Question q = questionRepository.findOne(questionId);
        User u = userSessionService.sessionUser().get();

        Optional<Vote> vote = likeService.getVote(u, q);
        if (vote.isPresent()) {
            likeService.undoLikeOrDislike(u, q);
            if (vote.get().isLike()) {
                likeService.addDislike(u, q);
            }
        } else {
            likeService.addDislike(u, q);
        }
        return getVoteStatus(q);
    }

    private VoteStatus getVoteStatus(Question question) {
        return new VoteStatus(likeService.getLikes(question), likeService.getDislikes(question));
    }


    @Value
    public static class VoteStatus {
        private final long likes;
        private final long dislikes;
    }

}
