package com.sharecare.webapp.service;

import com.sharecare.core.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.stereotype.Component;

import java.util.Arrays;

@Component
@RequiredArgsConstructor
public class AuthenticationProviderProxy implements AuthenticationProvider {

    private final UserService userService;

    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {

        boolean ok = userService.isAuthorized(authentication.getName(), (CharSequence) authentication.getCredentials());
        if (ok) {
            return new UsernamePasswordAuthenticationToken(
                    authentication.getName(),
                    null,
                    Arrays.asList(new SimpleGrantedAuthority("user"))
            );
        } else {
            throw new BadCredentialsException("Credentials are bad");
        }
    }
    @Override
    public boolean supports(Class<?> authentication) {
        return UsernamePasswordAuthenticationToken.class.isAssignableFrom(authentication);
    }

}
