package com.sharecare.webapp.service;

import com.sharecare.core.User;
import com.sharecare.core.UserRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * A service that allows easy access to the user's email and user's entity object of the current
 * contextual session. The service does not store any kind of session object, it just accesses
 * the Principal object holded by SecurityContext.
 */
@RequiredArgsConstructor
@Service
public class UserSessionService {

    private final UserRepository userRepository;

    public Optional<String> sessionUserEmail() {
        SecurityContext context = SecurityContextHolder.getContext();
        if (context != null) {
            Authentication authentication = context.getAuthentication();
            if (authentication != null) {
                String email = (String) authentication.getPrincipal();
                if (email != null && !email.isEmpty()) {
                    return Optional.of(email);
                }
            }
        }
        return Optional.empty();
    }

    public Optional<User> sessionUser() {
        return sessionUserEmail().map(userRepository::findByEmail);
    }

}